@@ -1,9 +1,10 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
+from trytond.pyson import Not, Bool, Eval
 
 STATES = {
-    'readonly': "active == False",
+    'readonly': Not(Bool(Eval('active'))),
 }
 
 
@@ -21,18 +22,18 @@ class Template(ModelSQL, ModelView):
         ], 'Type', required=True, states=STATES)
     category = fields.Many2One('product.category','Category', required=True,
             states=STATES)
-    list_price = fields.Property(type='numeric', string='List Price',
-            states=STATES, digits=(16, 4))
-    list_price_uom = fields.Function('get_price_uom', string='List Price',
-            type="numeric", digits=(16, 4))
-    cost_price = fields.Property(type='numeric', string='Cost Price',
-            states=STATES, digits=(16, 4))
-    cost_price_uom = fields.Function('get_price_uom', string='Cost Price',
-            type="numeric", digits=(16, 4), states=STATES)
-    cost_price_method = fields.Property(type='selection', selection=[
+    list_price = fields.Property(fields.Numeric('List Price', states=STATES,
+        digits=(16, 4)))
+    list_price_uom = fields.Function(fields.Numeric('List Price',
+        digits=(16, 4)), 'get_price_uom')
+    cost_price = fields.Property(fields.Numeric('Cost Price',
+            states=STATES, digits=(16, 4)))
+    cost_price_uom = fields.Function(fields.Numeric('Cost Price',
+        digits=(16, 4)), 'get_price_uom')
+    cost_price_method = fields.Property(fields.Selection([
         ("fixed", "Fixed"),
         ("average", "Average")
-        ], string="Cost Method", required=True, states=STATES)
+        ], 'Cost Method', required=True, states=STATES))
     default_uom = fields.Many2One('product.uom', 'Default UOM', required=True,
             states=STATES)
     active = fields.Boolean('Active', select=1)
@@ -48,7 +49,7 @@ class Template(ModelSQL, ModelView):
     def default_cost_price_method(self, cursor, user, context=None):
         return 'fixed'
 
-    def get_price_uom(self, cursor, user, ids, name, arg, context=None):
+    def get_price_uom(self, cursor, user, ids, name, context=None):
         product_uom_obj = self.pool.get('product.uom')
         res = {}
         if context is None:
@@ -88,7 +89,7 @@ class Product(ModelSQL, ModelView):
     code = fields.Char("Code", size=None, select=1)
     description = fields.Text("Description", translate=True)
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -99,18 +100,14 @@ class Product(ModelSQL, ModelView):
             res[product.id] = name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, [('code', args[i][1], args[i][2])],
-                    context=context)
-            if ids:
-                args2.append(('id', 'in', ids))
-            else:
-                args2.append(('name', args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, [('code',) + clause[1:]],
+                order=[], context=context)
+        if ids:
+            ids += self.search(cursor, user, [('name',) + clause[1:]],
+                    order=[], context=context)
+            return [('id', 'in', ids)]
+        return [('name',) + clause[1:]]
 
     def delete(self, cursor, user, ids, context=None):
         template_obj = self.pool.get('product.template')
